from resources.lib.api.api import API
from resources.lib.const import LANG, ROUTE, STRINGS, COMMAND, ACTION, DOWNLOAD_STATUS, MEDIA_TYPE, SETTINGS
from resources.lib.routing.router import router
from resources.lib.storage.sqlite import DB
from resources.lib.utils.kodiutils import get_string
from resources.lib.utils.url import Url
from resources.lib.storage.settings import settings


class ContextMenuRenderer:
    @staticmethod
    def run(url):
        return STRINGS.RUN_PLUGIN.format(url)

    @staticmethod
    def item(title, url):
        return tuple([title, STRINGS.RUN_PLUGIN.format(url)])

    @staticmethod
    def get_context_menu_history(route, media, *args):
        cmd = router.get_url(ROUTE.COMMAND, command=COMMAND.DELETE_HISTORY_ITEM, media_id=media.get('_id'))
        return ContextMenuRenderer.item(get_string(LANG.DELETE_ITEM_FROM_HISTORY), cmd)

    @staticmethod
    def pin(route, key, count_type, *args):
        route += count_type
        exists = DB.PINNED.exists(route, key)
        if exists:
            command = COMMAND.UNPIN
            lang = LANG.UNPIN
            title = STRINGS.UNPIN_ITEM.format(item=get_string(lang))
        else:
            command = COMMAND.PIN
            lang = LANG.PIN
            title = STRINGS.PIN_ITEM.format(item=get_string(lang))

        return ContextMenuRenderer.item(title, router.get_url(ROUTE.COMMAND, command=command, dir_route=route, key=key))

    @staticmethod
    def hide(route, key, *args):
        if key:
            return ContextMenuRenderer.item(get_string(LANG.HIDE),
                                            router.get_url(ROUTE.COMMAND, command=COMMAND.HIDE_MENU_ITEM,
                                                           dir_route=route, key=key))

    @staticmethod
    def show(route, key, dir_route, *args):
        return ContextMenuRenderer.item(get_string(LANG.MOVE_BACK),
                                        router.get_url(ROUTE.COMMAND, command=COMMAND.SHOW_MENU_ITEM,
                                                       dir_route=dir_route, key=key))

    @staticmethod
    def get_context_menu_download(route, media, url, *args):
        if settings[SETTINGS.DOWNLOADS_FOLDER]:
            media_id = media.get('_id')
            source = API.get_source(media)
            root_parent_id = source.get('root_parent', media.get('_id'))
            api_url = API.URL.streams(media.get('_id'))

            mediatype = source.get('info_labels').get("mediatype")

            if (mediatype == MEDIA_TYPE.MOVIE or mediatype == MEDIA_TYPE.EPISODE):
                return ContextMenuRenderer.item(get_string(LANG.DOWNLOAD),
                                                router.get_url(ROUTE.PROCESS_MEDIA_ITEM,
                                                               url=api_url,
                                                               root_parent_id=root_parent_id,
                                                               media_id=media_id,
                                                               action=ACTION.DOWNLOAD))

    @staticmethod
    def delete_download_item(route, media, *args):
        dl_id = media.get('dl_id')
        status = DB.DOWNLOAD.get_status(dl_id)
        if status == DOWNLOAD_STATUS.CANCELLED or status == DOWNLOAD_STATUS.COMPLETED:
            lang = LANG.DELETE
            command = COMMAND.DELETE_DOWNLOAD_ITEM
        else:
            lang = LANG.CANCEL_DOWNLOAD
            command = COMMAND.CANCEL_DOWNLOAD_ITEM

        return ContextMenuRenderer.item(get_string(lang),
                                        router.get_url(ROUTE.COMMAND,
                                                       command=command,
                                                       dl_id=dl_id))

    @staticmethod
    def start_download_item(route, media, url, *args):
        dl_id = media.get('dl_id')
        if DB.DOWNLOAD.status(dl_id, DOWNLOAD_STATUS.CANCELLED):
            return ContextMenuRenderer.get_context_menu_download(route, media, url)

    @staticmethod
    def pause_download_item(route, media, url, *args):
        dl_id = media.get('dl_id')
        status = DB.DOWNLOAD.get_status(dl_id)
        if status == DOWNLOAD_STATUS.DOWNLOADING or status == DOWNLOAD_STATUS.PAUSED or status == DOWNLOAD_STATUS.QUEUED:
            if status == DOWNLOAD_STATUS.DOWNLOADING or status == DOWNLOAD_STATUS.QUEUED:
                lang = LANG.PAUSE_DOWNLOAD
                cmd = COMMAND.PAUSE_DOWNLOAD_ITEM
            else:
                lang = LANG.RESUME_DOWNLOAD
                cmd = COMMAND.RESUME_DOWNLOAD_ITEM

            return ContextMenuRenderer.item(get_string(lang),
                                            router.get_url(ROUTE.COMMAND,
                                                           command=cmd,
                                                           dl_id=dl_id))

    @staticmethod
    def add_to_library_item(route, media, *args):
        if settings[SETTINGS.MOVIE_LIBRARY_FOLDER] or settings[SETTINGS.TV_SHOW_LIBRARY_FOLDER]:
            mediaid = media.get('_id')
            source = API.get_source(media)
            info_labels = source.get('info_labels')
            mediatype = info_labels.get("mediatype")
            if (mediatype == MEDIA_TYPE.MOVIE and settings[SETTINGS.MOVIE_LIBRARY_FOLDER]) or \
                    (mediatype == MEDIA_TYPE.TV_SHOW and settings[SETTINGS.TV_SHOW_LIBRARY_FOLDER]):
                lang = LANG.ADD_TO_LIBRARY
                cmd = COMMAND.ADD_TO_LIBRARY
                return ContextMenuRenderer.item(get_string(lang),
                                                router.get_url(ROUTE.COMMAND, command=cmd,
                                                               mediaid=mediaid,
                                                               mediatype=mediatype))

    @staticmethod
    def choose_stream(route, media, url, has_children):
        if not has_children:
            return ContextMenuRenderer.item(get_string(30404), url + '&force=1')

    @staticmethod
    def trakt(route, media, *args):
        source = API.get_source(media)
        services = source.get('services', {})
        trakt = services.get('trakt')
        info_labels = source.get('info_labels')
        media_type = info_labels.get("mediatype")
        if settings[SETTINGS.TRAKT_AUTHENTICATION] and trakt:
            return ContextMenuRenderer.item(
                get_string(LANG.TRAKT_MEDIA_MENU),
                router.get_url(ROUTE.COMMAND, command=COMMAND.TRAKT_MEDIA_MENU,
                               id=str(trakt),
                               media_type=media_type,
                               title=info_labels.get('sorttitle'))
            )
